package edu.mit.csail.pag.recrash.generator;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import edu.mit.csail.pag.recrash.GetClass;

/**
 * a proxy to invoke non public class
 * 
 * @author hunkim
 * 
 */
public class ProxyObject {

	private Object object;

	Class<?> clazz = null;

	/**
	 * Get object to invoke
	 * 
	 * @param object
	 */
	public ProxyObject(Object object) {
		this.object = object;
		clazz = object.getClass();
	}

	/**
	 * Call corresponding method based on the arguments
	 * 
	 * @param methodName
	 * @param arguments
	 * @return
	 * @throws SecurityException
	 * @throws NoSuchMethodException
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws ClassNotFoundException
	 */
	public Object call(String methodName, String argClassNames[],
			Object... arguments) throws SecurityException,
			NoSuchMethodException, IllegalArgumentException,
			IllegalAccessException, InvocationTargetException,
			ClassNotFoundException {

		assert (argClassNames.length == arguments.length);

		Class<?> clazzes[] = new Class[arguments.length];

		for (int i = 0; i < argClassNames.length; i++) {
			clazzes[i] = GetClass.fromTypeName(argClassNames[i]);
		}

		Method methods[] = clazz.getMethods();
		for (Method method : methods) {
			if (method.getName().equals(methodName)) {
				System.out.println("Method: " + method);
			}
		}

		Method method = clazz.getMethod(methodName, clazzes);
		method.setAccessible(true);
		return method.invoke(object, arguments);
	}

	/**
	 * Call corresponding method based on the arguments
	 * 
	 * @param methodName
	 * @param arguments
	 * @return
	 * @throws SecurityException
	 * @throws NoSuchMethodException
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws ClassNotFoundException
	 */
	public Object callOld(String methodName, String argClassNames[],
			Object... arguments) throws SecurityException,
			NoSuchMethodException, IllegalArgumentException,
			IllegalAccessException, InvocationTargetException,
			ClassNotFoundException {

		assert (argClassNames.length == arguments.length);

		Class<?> clazzes[] = new Class[arguments.length];

		for (int i = 0; i < argClassNames.length; i++) {
			clazzes[i] = GetClass.fromTypeName(argClassNames[i]);
		}

		Method methods[] = clazz.getMethods();
		for (Method method : methods) {
			if (method.getName().equals(methodName)) {
				System.out.println("Method: " + method);
			}
		}

		Method method = clazz.getMethod(methodName, clazzes);
		method.setAccessible(true);
		return method.invoke(object, arguments);
	}

	public static void main(String args[]) throws SecurityException,
			IllegalArgumentException, NoSuchMethodException,
			IllegalAccessException, InvocationTargetException,
			ClassNotFoundException {
		System.out.println(int[].class + ":" + Class.forName("I"));
		String test = "Hello";
		System.out.println(new ProxyObject(test).call("toString",
				new String[] {}));
	}
}
