package edu.mit.csail.pag.recrash.tracer;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Date;
import java.util.zip.GZIPInputStream;

import checkers.quals.*;

import edu.mit.csail.pag.recrash.Util;
import edu.mit.csail.pag.recrash.tracer.item.MethodTraceItem;
import edu.mit.csail.pag.recrash.tracer.item.ClonedArgumentObject;
import edu.mit.csail.pag.recrash.tracer.item.StackTraceItem;

/**
 * Read a trace from a trace file
 * 
 * @author hunkim
 * 
 */
/*@Default("checkers.quals.NonNull")*/

public class TraceReader {

	static private MethodTraceItem traceItem = null;

	static StackTraceItem stackTraceItem = null;

	static Date traceFileDate;

	public static void setStackTraceItem(StackTraceItem stackTraceItem) {
		TraceReader.stackTraceItem = stackTraceItem;
	}

	public static void setMethodTraceItem(int order) {
		if (stackTraceItem != null && order >= 0
				&& order < stackTraceItem.size()) {
			traceItem = stackTraceItem.get(order);
		}
	}

	/**
	 * Read traceItem from serialized information in file
	 * 
	 */
	public static StackTraceItem readTrace() {
		return readTrace(Util.TRACE_FILE);
	}

	/**
	 * Read traceItem from serialized information in file
	 * 
	 */
	public static StackTraceItem readTrace(String traceFileName) {
		stackTraceItem = null;
		try {

			File traceFile = new File(traceFileName);
			traceFileDate = new Date(traceFile.lastModified());

			Reader fr = null;

			if (traceFileName.endsWith(".gz")) {
				fr = new InputStreamReader(new GZIPInputStream(
						new FileInputStream(traceFile)));
			} else {
				fr = new FileReader(traceFileName);
			}

			stackTraceItem = (StackTraceItem) Serializer.loadObject(fr);
			//System.gc();
			fr.close();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return stackTraceItem;
	}

	/**
	 * Get object by name from trace
	 * 
	 * @param name
	 * @return
	 */
	public static Object readObject(int index) {
		if (traceItem == null) {
			return null;
		}
		ClonedArgumentObject argumentObject = traceItem
				.getArgumentObject(index);
		if (argumentObject == null) {
			return null;
		}

		return argumentObject.getObject();
	}

	/**
	 * Get static object by name from trace
	 * 
	 * @param name
	 * @return
	 */
	public static /*@Nullable*/ Object readStatic(String className) {
		if (traceItem == null) {
			return null;
		}
		String serealizedString = traceItem.readStatic(className);

		// No object found or the object was null
		if (serealizedString == null) {
			return null;
		}

		return Serializer.loadObject(serealizedString);
	}

	public static Date getTraceFileDate() {
		return traceFileDate;
	}
}
