package edu.mit.csail.pag.recrash.tracer.item;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import checkers.quals.*;

import edu.mit.csail.pag.recrash.GetClass;
import edu.mit.csail.pag.recrash.Logger;
import edu.mit.csail.pag.recrash.tracer.Serializer;

/*@Default("checkers.quals.NonNull")*/

public class ClonedArgumentObject {
	private static int count = 0;
	private boolean isCloned;

	private String typeName;

	/* @Nullable */
	private Object object;

	/* @Nullable */
	private String serializedString;

	public ClonedArgumentObject(Object object, String classTypeName,
			boolean needToBeCloned) {

		this.typeName = classTypeName;
		this.object = object;
		if (object == null)
			return;

		if (needToBeCloned) {
			serializedString = Serializer.toXMLString(object);
			this.object = null; // we serialize it
			if (Logger.verbose) {
				System.out.print('.');
				if (count++ % 100 == 0) {
					System.out.print("\n" + count);
				}
			}
		}

		if (true) {
			return;
		}

		// ## FIXME: we don't need to use this
		// the clone will be called in the instrumentation
		if (!needToBeCloned || object == null) {
			this.object = object;
			return;
		}

		/* @Nullable */Class<?> clazz = object.getClass();
		if (clazz == null) {
			return;
		}

		try {
			Method method = clazz.getDeclaredMethod("clone");
			method.setAccessible(true);
			this.object = method.invoke(object);
			this.isCloned = true;

			Logger.info("!!! Cloned: " + classTypeName);

		} catch (Exception e) {
			e.printStackTrace();
			Logger.fatal("Cannot clone for : " + clazz.getName());
			System.exit(0);
			this.object = object;
		}
	}

	/**
	 * Get this object type name. If it is a inner class, we change it to '."
	 * 
	 * @return
	 */
	public String getTypeName() {
		return typeName;
	}

	public Object getObject() {
		if (serializedString != null) {
			object = Serializer.loadObject(serializedString);
			serializedString = null;
		}
		return object;
	}

	/**
	 * Is this argument object public
	 * 
	 * @return
	 */
	public boolean isPublic() {
		try {
			int modifiers = GetClass.fromTypeName(typeName).getModifiers();
			return (modifiers & Modifier.PUBLIC) == 1;
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return false;
	}

	public boolean isCloned() {
		return isCloned;
	}
}
