package edu.mit.csail.pag.recrash.transformation;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.LinkedHashSet;
import java.util.Set;

import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.MethodNode;

import edu.mit.csail.pag.recrash.GetClass;
import edu.mit.csail.pag.recrash.Logger;
import edu.mit.csail.pag.recrash.Util;

public class Mutability {

	Set<String> immutableSet = new LinkedHashSet<String>();

	public Mutability(String immutableParametersPath) throws IOException {
		BufferedReader br = new BufferedReader(new FileReader(
				immutableParametersPath));
		while (true) {
			String line = br.readLine();
			if (line == null) {
				break;
			}

			String splits[] = line.split("\\s+");
			if (splits.length < 2) {
				continue;
			}
			String key = makeKey(splits[0], splits[1]);
			immutableSet.add(key);
		}
		Logger
				.info(immutableSet.size()
						+ " immutable signature/pos are added!");
		br.close();
	}

	/**
	 * Make a key from methodSig and position
	 * 
	 * @param string
	 * @param string2
	 * @return
	 */
	private String makeKey(String signature, String pos) { // Don't add
		// instrumentation
		// inside of the
		// clone
		return Util.transClassNameSlashToDot(pos + "@" + signature);
	}

	public boolean isImmutable(MethodNode mn, ClassNode cn, String pos) {
		String methodSignature;
		try {
			methodSignature = cn.name + "." + mn.name + "("
					+ GetClass.descToSig(mn.desc) + ")";
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		String key = makeKey(methodSignature, pos);

		Logger.debug("[IM] " + key);
		return immutableSet.contains(key);
	}
}
